'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _bluebird;

function _load_bluebird() {
  return _bluebird = require('bluebird');
}

var _bluebird2;

function _load_bluebird2() {
  return _bluebird2 = _interopRequireDefault(require('bluebird'));
}

let addFileToAssets = (() => {
  var _ref = (0, (_bluebird || _load_bluebird()).coroutine)(function* (compilation, htmlPluginData, _ref2) {
    let filepath = _ref2.filepath;
    var _ref2$typeOfAsset = _ref2.typeOfAsset;
    let typeOfAsset = _ref2$typeOfAsset === undefined ? 'js' : _ref2$typeOfAsset;
    var _ref2$includeSourcema = _ref2.includeSourcemap;
    let includeSourcemap = _ref2$includeSourcema === undefined ? true : _ref2$includeSourcema;
    var _ref2$hash = _ref2.hash;
    let hash = _ref2$hash === undefined ? false : _ref2$hash,
        publicPath = _ref2.publicPath,
        outputPath = _ref2.outputPath;
    var _ref2$files = _ref2.files;
    let files = _ref2$files === undefined ? [] : _ref2$files;

    if (!filepath) {
      const error = new Error('No filepath defined');
      compilation.errors.push(error);
      return (_bluebird2 || _load_bluebird2()).default.reject(error);
    }

    const filters = Array.isArray(files) ? files : [files];

    if (filters.length > 0) {
      const shouldSkip = !files.some(function (file) {
        return (0, (_minimatch || _load_minimatch()).default)(htmlPluginData.outputName, file);
      });

      if (shouldSkip) {
        return (_bluebird2 || _load_bluebird2()).default.resolve(null);
      }
    }

    const addedFilename = yield htmlPluginData.plugin.addFileToAssets(filepath, compilation);

    let suffix = '';
    if (hash) {
      const md5 = (_crypto || _load_crypto()).default.createHash('md5');
      md5.update(compilation.assets[addedFilename].source());
      suffix = `?${md5.digest('hex').substr(0, 20)}`;
    }

    const resolvedPublicPath = typeof publicPath === 'undefined' ? resolvePublicPath(compilation, addedFilename) : ensureTrailingSlash(publicPath);
    const resolvedPath = `${resolvedPublicPath}${addedFilename}${suffix}`;

    htmlPluginData.assets[typeOfAsset].unshift(resolvedPath);

    resolveOutput(compilation, addedFilename, outputPath);

    if (includeSourcemap) {
      const addedMapFilename = yield htmlPluginData.plugin.addFileToAssets(`${filepath}.map`, compilation);
      resolveOutput(compilation, addedMapFilename, outputPath);
    }

    return (_bluebird2 || _load_bluebird2()).default.resolve(null);
  });

  return function addFileToAssets(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
})();

// Visible for testing


var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _crypto;

function _load_crypto() {
  return _crypto = _interopRequireDefault(require('crypto'));
}

var _minimatch;

function _load_minimatch() {
  return _minimatch = _interopRequireDefault(require('minimatch'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ensureTrailingSlash(string) {
  if (string.length && string.substr(-1, 1) !== '/') {
    return `${string}/`;
  }

  return string;
}

// Copied from html-webpack-plugin
function resolvePublicPath(compilation, filename) {
  /* istanbul ignore else */
  const publicPath = typeof compilation.options.output.publicPath !== 'undefined' ? compilation.options.output.publicPath : (_path || _load_path()).default.relative((_path || _load_path()).default.dirname(filename), '.'); // TODO: How to test this? I haven't written this logic, unsure what it does

  return ensureTrailingSlash(publicPath);
}

function resolveOutput(compilation, addedFilename, outputPath) {
  if (outputPath && outputPath.length) {
    compilation.assets[`${outputPath}/${addedFilename}`] = compilation.assets[addedFilename]; // eslint-disable-line no-param-reassign
    delete compilation.assets[addedFilename]; // eslint-disable-line no-param-reassign
  }
}

exports.default = (() => {
  var _ref3 = (0, (_bluebird || _load_bluebird()).coroutine)(function* (assets, compilation, htmlPluginData, callback) {
    try {
      yield (_bluebird2 || _load_bluebird2()).default.mapSeries(assets, function (asset) {
        return addFileToAssets(compilation, htmlPluginData, asset);
      });

      callback(null, htmlPluginData);
    } catch (e) {
      callback(e, htmlPluginData);
    }
  });

  return function (_x4, _x5, _x6, _x7) {
    return _ref3.apply(this, arguments);
  };
})();

module.exports = exports['default'];